import 'package:circlegraph/circlegraph.dart';
import 'package:dashboard/loading.dart';
import 'package:dashboard/querying/communitygetter.dart';
import 'package:dashboard/ui/theme/color_holder.dart';
import 'package:dashboard/ui/util/logged_in_appbar.dart';
import 'package:enhanced_future_builder/enhanced_future_builder.dart';
import 'package:flutter/cupertino.dart';

import 'package:flutter/material.dart';
import 'package:flutter_spinkit/flutter_spinkit.dart';
import '../data/post.dart';
import '../data/userGraphNodeData.dart';
import 'usernode.dart';
import 'postnode.dart';
import '../data/user.dart';

class UserGraph extends StatefulWidget {
  UserGraph(this.user);
  final User user;

  @override
  _UserGraphState createState() => _UserGraphState(user);
}

class _UserGraphState extends State<UserGraph> {
  _UserGraphState(this.user);
  final User user;

  int nrPerPage = 15;
  int pageNr = 0;

  Widget build(BuildContext context) {
    List<Post> nodes = [];

    for (int i = pageNr * nrPerPage;
        (i < pageNr * nrPerPage + nrPerPage) && i < user.posts.length;
        i++) {
      nodes.add(user.posts[i]);
    }

    return Container(
      //constraints: BoxConstraints.expand(),
      child: SingleChildScrollView(
        child: Center(
          child: CircleGraph(
            root: _getUserNode(user),
            children: [for (Post post in nodes) _getPostNode(post)],
            edgeColor: Colors.black,
            backgroundColor: ColorHolder.color3,
          ),
        ),
      ),
    );
  }

  TreeNodeData<UserGraphNodeData> _getUserNode(User user) {
    UserGraphNodeData data = UserGraphNodeData.fromUser(user);

    return TreeNodeData(
        child: UserNode(user.trustScores),
        data: data,
        backgroundColor: Colors.transparent);
  }

  TreeNodeData<UserGraphNodeData> _getPostNode(Post post) {
    UserGraphNodeData data = UserGraphNodeData.fromPost(post);

    return TreeNodeData(
      child: PostNode(post),
      data: data,
      width: 250,
    );
  }
}

class UserGraphViewPage extends StatelessWidget {
  UserGraphViewPage(this.username);
  final String username;

  Widget build(BuildContext context) {
    return Scaffold(
        backgroundColor: ColorHolder.color3,
        appBar: LoggedInAppBar("User: $username"),
        body: Column(
          mainAxisSize: MainAxisSize.max,
          children: [
            EnhancedFutureBuilder<User>(
              future: DataGetter.getUser(username),
              // future: DataGetter.getJson(
              //     "http://live.nope-api.systems:81/users/" +
              //         username.toString()),
              rememberFutureResult: false,
              whenDone: (result) => UserGraph(result),
              whenNotDone: Center(
                child: Column(
                  mainAxisSize: MainAxisSize.min,
                  children: [
                    SpinKitRotatingCircle(
                      color: ColorHolder.color2,
                      size: 200.0,
                    ),
                    SizedBox(height: 16),
                    Text(
                      "Loading",
                      style: TextStyle(
                        color: ColorHolder.color2,
                        fontSize: 22,
                      ),
                    ),
                  ],
                ),
              ),
            ),
          ],
        ));
  }
}
