import 'package:dashboard/data/cluster.dart';
import 'package:dashboard/data/community.dart';
import 'package:dashboard/data/post.dart';
import 'package:dashboard/querying/communitygetter.dart';
import 'package:dashboard/querying/host_provider.dart';
import 'package:dashboard/util/loginhelper.dart';
import 'package:dashboard/util/user.dart';
import 'package:logger/logger.dart';

class ClusterGetter {
  Map<String, List<Community>> clusterMap;

  Future<List<Community>> getNodesfromLayer(String layerName) async {
    String url =
        "${HostProvider.articonfHost}/api/use-cases/reddit/tables/reddit/layers/$layerName/nodes";

    BackendUser user = await LoginHelper().loggedInUser;
    String token = user.token;

    Map<String, dynamic> headers = {"Authorization": "Bearer $token"};

    List<dynamic> nodeList =
        await DataGetter.getJsonList(url, headers: headers);

    List<Community> communityList = [];

    for (Map<String, dynamic> entry in nodeList) {
      bool communityExists = false;
      Community community;
      for (Community tmpCommunity in communityList) {
        if (tmpCommunity.name == entry["subreddit"]) {
          community = tmpCommunity;
          communityExists = true;
          break;
        }
      }

      if (!communityExists) {
        community = Community(entry["subreddit"], 1, []);
        communityList.add(community);
      }

      community.posts.add(Post(
          entry["title"],
          entry["content"],
          entry["user_id"],
          entry["subreddit"],
          entry["percentage_upvoted"],
          entry["n_comments"]));
    }

    return communityList;
  }

  Future<Map<String, List<Community>>> getClustersFuture() async {
    List<String> layers = [
      "Upvotes_Layer",
      "Percentage_Layer",
      "Engagement_Layer",
      "Time_Layer",
    ];

    Map<String, List<Community>> clusterMap = {};
    for (String layer in layers) {
      clusterMap[layer] = await getNodesfromLayer(layer);
    }
    Logger().i(clusterMap.toString());
    return clusterMap;
  }

  Future<List<Cluster>> getClustersFromLayer(String layerName) async {
    String url =
        "${HostProvider.articonfHost}/api/use-cases/reddit/tables/reddit/layers/$layerName/clusters";

    BackendUser user = await LoginHelper().loggedInUser;
    String token = user.token;

    Map<String, dynamic> headers = {"Authorization": "Bearer $token"};

    List<dynamic> clusterMapList =
        await DataGetter.getJsonList(url, headers: headers);

    List<Cluster> clusterList = [];
    for (Map<String, dynamic> clusterMap in clusterMapList) {
      Cluster cluster = Cluster(clusterMap["label"],
          clusterMap["cluster_label"], clusterMap["layer_name"], []);

      for (Map<String, dynamic> nodeMap in clusterMap["nodes"]) {
        cluster.nodes.add(Post(
            nodeMap["title"],
            nodeMap["content"],
            nodeMap["user_id"],
            nodeMap["subreddit"],
            nodeMap["percentage_upvoted"],
            nodeMap["n_comments"]));
      }

      clusterList.add(cluster);
    }

    return clusterList;
  }

  Future<Cluster> getSpecificClusterFromLayer(String layerName, int id) async {
    List<Cluster> clusterList = await getClustersFromLayer(layerName);

    for (Cluster cluster in clusterList) {
      if (cluster.label == id) return cluster;
    }
    return null;
  }
}
